#include <LiquidCrystal_PCF8574.h>
#include <SoftwareSerial.h>
#include <Wire.h>

// RX, TX pins for SoftwareSerial
SoftwareSerial mySerial(14, 15);

// Define motor pins

// Motor 1 - Back Right
int motor1pin1 = 2;
int motor1pin2 = 3;
int motor1pwm = 5;

// Motor 2 - Back Left
int motor2pin1 = 4;
int motor2pin2 = 7;
int motor2pwm = 6;

// Motor 3 - Front Left
int motor3pin1 = 8;
int motor3pin2 = 11;
int motor3pwm = 9;

// Motor 4 - Front Right
int motor4pin1 = 12;
int motor4pin2 = 13;
int motor4pwm = 10;

int motorSpeed1 = 90;  // Adjust the speed as needed
int motorSpeed2 = 86;  // Adjust the speed as needed
int motorSpeed3 = 79;  // Adjust the speed as needed
int motorSpeed4 = 72;  // Adjust the speed as needed

// Define LCD I2C address and characteristics
#define LCD_ADDRESS 0x27  // Updated with your found address
#define LCD_ROWS 2
#define LCD_COLUMNS 16

// LED pin
#define LED_PIN 13  // Assuming built-in LED on Arduino Uno

// Initialize the LCD
LiquidCrystal_PCF8574 lcd(LCD_ADDRESS);

void setup() {
  // Initialize the Serial Monitor for debugging
  Serial.begin(9600);
  mySerial.begin(115200);

  // print motor speeds
  Serial.print("Motor Speeds: ");
  Serial.print(motorSpeed1);
  Serial.print(", ");
  Serial.print(motorSpeed2);
  Serial.print(", ");
  Serial.print(motorSpeed3);
  Serial.print(", ");
  Serial.println(motorSpeed4);

  while (!mySerial);  // Wait for serial port to connect. Needed for native USB

  // Initialize the LCD
  lcd.begin(LCD_COLUMNS, LCD_ROWS);
  lcd.setBacklight(255);  // Turn on the backlight

  // Set LED pin as output
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, HIGH);

  // Print initial message on LCD
  lcd.setCursor(0, 0);
  lcd.print("WAITING TO");
  lcd.setCursor(0, 1);
  lcd.print("CONNECT");

  // Setup motor pins
  pinMode(motor1pin1, OUTPUT);
  pinMode(motor1pin2, OUTPUT);
  pinMode(motor1pwm, OUTPUT);

  pinMode(motor2pin1, OUTPUT);
  pinMode(motor2pin2, OUTPUT);
  pinMode(motor2pwm, OUTPUT);

  pinMode(motor3pin1, OUTPUT);
  pinMode(motor3pin2, OUTPUT);
  pinMode(motor3pwm, OUTPUT);

  pinMode(motor4pin1, OUTPUT);
  pinMode(motor4pin2, OUTPUT);
  pinMode(motor4pwm, OUTPUT);
}

void loop() {
  static unsigned long lastReceiveTime = 0;
  static String lastCommand = "";  // Keep track of the last command sent
  static bool noResponse = false;

  if (mySerial.available()) {
    String receivedMessage = mySerial.readStringUntil('\n');
    lastReceiveTime = millis();

    digitalWrite(LED_PIN, LOW);
    delay(200);
    digitalWrite(LED_PIN, HIGH);

    if (handleCommand(receivedMessage)) {
      mySerial.println("ACK");  // Send acknowledgment if command is handled
    } else {
      mySerial.println("NACK");  // Request resend if command is invalid
    }

    lastCommand = receivedMessage;  // Update last command
    noResponse = false;
  }

  // Check for no response and request resend
  if (millis() - lastReceiveTime > 1000 && !noResponse) {
    noResponse = true;
    mySerial.println("RESEND");
    Serial.println("RESEND");
  }
}

bool handleCommand(String command) {
  command.trim();  // Clean up command string

  // Motor commands
  if (command.startsWith("cmd ")) {
    String direction = command.substring(4);
    if (executeDirection(direction)) {
      return true;  // Return true if command is executed
    }
  }
  // Print the IP address on the LCD
  else if (command.startsWith("Cam")) {
    // print the message received, split it on the two lines
    int len = command.length();

    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(command.substring(0, 16));
    if (len > 16) {
      lcd.setCursor(0, 1);
      lcd.print(command.substring(16, len));
      // clear the rest of the line
      for (int i = len; i < 32; i++) {
        lcd.print(" ");
      }
    } else {
      // clear the rest of the line
      for (int i = len; i < 16; i++) {
        lcd.print(" ");
      }
    }

    return true;  // Return true if command is executed
  }

  return false;  // Return false if command is unknown
}

bool executeDirection(String direction) {
  // Motor commands
  if (direction == "forward") {
    moveForward();
  } else if (direction == "backward") {
    moveBackward();
  } else if (direction == "left") {
    moveLeft();
  } else if (direction == "right") {
    moveRight();
  } else if (direction == "front_left") {
    moveDiagonalFrontLeft();
  } else if (direction == "front_right") {
    moveDiagonalFrontRight();
  } else if (direction == "back_left") {
    moveDiagonalBackLeft();
  } else if (direction == "back_right") {
    moveDiagonalBackRight();
  } else if (direction == "rotate_left") {
    rotateLeft();
  } else if (direction == "rotate_right") {
    rotateRight();
  } else if (direction == "stop") {
    stop();
  } else {
    Serial.println("Unknown direction: " + direction);
    return false;
  }
  return true;
}

void moveBackward() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor1pin1, HIGH);
  digitalWrite(motor1pin2, LOW);
  digitalWrite(motor2pin1, HIGH);
  digitalWrite(motor2pin2, LOW);
  digitalWrite(motor3pin1, LOW);
  digitalWrite(motor3pin2, HIGH);
  digitalWrite(motor4pin1, LOW);
  digitalWrite(motor4pin2, HIGH);
}

void moveForward() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, HIGH);
  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, HIGH);
  digitalWrite(motor3pin1, HIGH);
  digitalWrite(motor3pin2, LOW);
  digitalWrite(motor4pin1, HIGH);
  digitalWrite(motor4pin2, LOW);
}

void moveRight() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, HIGH);
  digitalWrite(motor2pin1, HIGH);
  digitalWrite(motor2pin2, LOW);
  digitalWrite(motor3pin1, HIGH);
  digitalWrite(motor3pin2, LOW);
  digitalWrite(motor4pin1, LOW);
  digitalWrite(motor4pin2, HIGH);
}

void moveLeft() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor1pin1, HIGH);
  digitalWrite(motor1pin2, LOW);
  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, HIGH);
  digitalWrite(motor3pin1, LOW);
  digitalWrite(motor3pin2, HIGH);
  digitalWrite(motor4pin1, HIGH);
  digitalWrite(motor4pin2, LOW);
}

void moveDiagonalFrontRight() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, 0);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, 0);

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, HIGH);
  digitalWrite(motor3pin1, HIGH);
  digitalWrite(motor3pin2, LOW);

  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, LOW);
  digitalWrite(motor4pin1, LOW);
  digitalWrite(motor4pin2, LOW);
}

void moveDiagonalFrontLeft() {
  analogWrite(motor1pwm, 0);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, 0);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, HIGH);
  digitalWrite(motor4pin1, HIGH);
  digitalWrite(motor4pin2, LOW);

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, LOW);
  digitalWrite(motor3pin1, LOW);
  digitalWrite(motor3pin2, LOW);
}

void moveDiagonalBackLeft() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, 0);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, 0);

  digitalWrite(motor1pin1, HIGH);
  digitalWrite(motor1pin2, LOW);
  digitalWrite(motor3pin1, LOW);
  digitalWrite(motor3pin2, HIGH);

  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, LOW);
  digitalWrite(motor4pin1, LOW);
  digitalWrite(motor4pin2, LOW);
}

void moveDiagonalBackRight() {
  analogWrite(motor1pwm, 0);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, 0);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor2pin1, HIGH);
  digitalWrite(motor2pin2, LOW);
  digitalWrite(motor4pin1, LOW);
  digitalWrite(motor4pin2, HIGH);

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, LOW);
  digitalWrite(motor3pin1, LOW);
  digitalWrite(motor3pin2, LOW);
}

void rotateLeft() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, HIGH);
  digitalWrite(motor2pin1, HIGH);
  digitalWrite(motor2pin2, LOW);
  digitalWrite(motor3pin1, LOW);
  digitalWrite(motor3pin2, HIGH);
  digitalWrite(motor4pin1, HIGH);
  digitalWrite(motor4pin2, LOW);
}

void rotateRight() {
  analogWrite(motor1pwm, motorSpeed1);
  analogWrite(motor2pwm, motorSpeed2);
  analogWrite(motor3pwm, motorSpeed3);
  analogWrite(motor4pwm, motorSpeed4);

  digitalWrite(motor1pin1, HIGH);
  digitalWrite(motor1pin2, LOW);
  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, HIGH);
  digitalWrite(motor3pin1, HIGH);
  digitalWrite(motor3pin2, LOW);
  digitalWrite(motor4pin1, LOW);
  digitalWrite(motor4pin2, HIGH);
}

void stop() {
  analogWrite(motor1pwm, 0);
  analogWrite(motor2pwm, 0);
  analogWrite(motor3pwm, 0);
  analogWrite(motor4pwm, 0);

  digitalWrite(motor1pin1, LOW);
  digitalWrite(motor1pin2, LOW);
  digitalWrite(motor2pin1, LOW);
  digitalWrite(motor2pin2, LOW);
  digitalWrite(motor3pin1, LOW);
  digitalWrite(motor3pin2, LOW);
  digitalWrite(motor4pin1, LOW);
  digitalWrite(motor4pin2, LOW);
}